"""

    """

import asyncio

from src.mirutil.async_req import get_reqs_and_save_async_sync as grasas


##
url = 'https://excel.codal.ir/service/Excel/GetAll/3scwkK3cwWVPTvP5QQQaQQQ2%2b9RA%3d%3d/0'

grasas([url] , ['1.html'])

##

##

##

##



"""

    """

import re
from dataclasses import dataclass


@dataclass
class RFindFn :
    fn: (str , None)
    suf: (str , None)

def find_fn_and_suf_fr_codal_get_resp(r_header) :
    cd = r_header["Content-Disposition"]

    pat = 'filename=(.+)\.(\w+)'
    rf = re.findall(pat , cd)
    if len(rf) == 0 :
        return RFindFn(None , None)

    g = rf[0]

    return RFindFn(fn = g[0] , suf = g[1])



"""

    """

import asyncio
from dataclasses import dataclass
from functools import partial

import nest_asyncio
from aiohttp import ClientSession
from aiohttp.client_exceptions import ClientConnectorError
from aiohttp.client_exceptions import ClientOSError
from aiohttp.client_exceptions import ClientPayloadError
from aiohttp import ClientResponse
from asyncio.exceptions import TimeoutError

from .const import Const
from .files import write_to_file_async


nest_asyncio.apply()

cte = Const()

@dataclass
class RGet :
    r: ClientResponse | None = None
    exc: str | None = None
    cont: bytes | None = None

async def _get_a_req_async(url ,
                           client_session ,
                           headers = cte.headers ,
                           params = None ,
                           ssl = True ,
                           timeout = None) :
    try :
        r = await client_session.get(url ,
                                     headers = headers ,
                                     params = params ,
                                     ssl = ssl ,
                                     timeout = timeout)
        return RGet(r = r)
    except (ClientConnectorError , ClientPayloadError , ClientOSError ,
            TimeoutError) as e :
        print(e)
        return RGet(exc = str(e))

async def _process_rget(rget , mode) :
    if rget.exc is not None :
        return rget

    if rget.r.status != 200 :
        return rget

    if mode == 'read' :
        rget.cont = await rget.r.read()
    elif mode == 'json' :
        rget.cont = await rget.r.json()
    return rget

async def _get_resps_async(urls , mode , **kwargs) :
    sess = ClientSession()
    f = partial(_get_a_req_async , client_session = sess , **kwargs)
    co_tasks = [f(x) for x in urls]
    resps = await asyncio.gather(*co_tasks)
    f1 = partial(_process_rget , mode = mode)
    o = await asyncio.gather(*[f1(x) for x in resps])
    await sess.close()
    return o

def get_resps_async_sync(urls , mode = 'read' , **kwargs) :
    return asyncio.run(_get_resps_async(urls , mode = mode , **kwargs))

async def _get_a_req_and_save_async(url ,
                                    fp ,
                                    client_session ,
                                    mode ,
                                    write_mode ,
                                    encoding ,
                                    get_timeout ,
                                    **kwargs) :
    o = await _get_a_req_async(url ,
                               client_session ,
                               timeout = get_timeout ,
                               **kwargs)
    if o.exc is not None :
        return o

    if o.r.status == 200 :
        o = await _process_rget(o , mode = mode)
        await write_to_file_async(o.cont , fp , write_mode , encoding)

    return o

async def _get_reqs_and_save_async(urls ,
                                   fps ,
                                   mode ,
                                   write_mode ,
                                   encoding ,
                                   get_timeout ,
                                   **kwargs) :
    cs = ClientSession()
    f = partial(_get_a_req_and_save_async ,
                client_session = cs ,
                mode = mode ,
                write_mode = write_mode ,
                encoding = encoding ,
                get_timeout = get_timeout ,
                **kwargs)
    co_tasks = [f(x , y) for x , y in zip(urls , fps)]
    o = await asyncio.gather(*co_tasks)
    await cs.close()
    return o

def get_reqs_and_save_async_sync(urls ,
                                 fps ,
                                 mode = 'read' ,
                                 write_mode = 'w' ,
                                 encoding = 'utf-8' ,
                                 get_timeout = 10 ,
                                 **kwargs) :
    return asyncio.run(_get_reqs_and_save_async(urls ,
                                                fps ,
                                                mode = mode ,
                                                write_mode = write_mode ,
                                                encoding = encoding ,
                                                get_timeout = get_timeout ,
                                                **kwargs))



"""

    """

import pandas as pd
from html_table_parser.parser import HTMLTableParser
from lxml import etree


hdn_xpth = {
        "//*[@hidden]"                          : None ,
        '//*[contains(@style, "display:none")]' : None ,
        '//*[@class="non-visible-first"]'       : None ,
        }

def make_hidden_elements_of_etree_row_col_span_zero(tree: etree) -> etree :
    for xp in hdn_xpth.keys() :
        for el in tree.xpath(xp) :
            el.set("rowspan" , "0")
            el.set("colspan" , "0")
    return tree

def rm_hidden_elements_of_etree(tree: etree) -> etree :
    for xp in hdn_xpth.keys() :
        for el in tree.xpath(xp) :
            el.getparent().remove(el)

    return tree

def parse_html_as_etree(html: str) -> etree :
    parser = etree.HTMLParser()
    tree = etree.fromstring(html , parser)
    return tree

def etree_to_html(tree: etree) -> str :
    return etree.tostring(tree , encoding = "unicode" , method = "html")

def read_tables_in_html_by_html_table_parser(html: str) -> list :
    p = HTMLTableParser()
    p.feed(html)
    dfs = [pd.DataFrame(x) for x in p.tables]
    return dfs



"""

    """

import asyncio
from dataclasses import dataclass
from functools import partial

import nest_asyncio
from lxml.etree import XMLSyntaxError
from pyppeteer.errors import PageError
from pyppeteer.errors import TimeoutError as tout
from requests.exceptions import ConnectionError
from requests.exceptions import ReadTimeout as rtout
from requests_html import AsyncHTMLSession
from requests_html import HTMLSession

from .const import Const
from .files import write_txt_to_file_async


nest_asyncio.apply()

cte = Const()
ases = AsyncHTMLSession()

def download_chromium_if_not_installed(url = 'https://www.google.com') :
    """ downloads chromium if it is not installed """
    s = HTMLSession()
    r = s.get(url , headers = cte.headers , timeout = 10)
    s.close()
    r.html.render(timeout = 30)

@dataclass
class RGetAndRender :
    status: int
    headers: dict
    html: (str , None) = None
    err: (str , None) = None

async def get_a_req_and_render_by_requests_html_async(url ,
                                                      headers = cte.headers ,
                                                      params = None ,
                                                      verify = True ,
                                                      get_timeout = None ,
                                                      render_timeout = None) :
    r = await ases.get(url ,
                       headers = headers ,
                       params = params ,
                       verify = verify ,
                       timeout = get_timeout)
    try :
        await r.html.arender(timeout = render_timeout)
        return RGetAndRender(status = r.status_code ,
                             headers = r.headers ,
                             html = r.html.html ,
                             err = None)
    except (XMLSyntaxError , tout , PageError , ConnectionError , rtout) as e :
        print(e)
        return RGetAndRender(status = r.status_code ,
                             headers = r.headers ,
                             html = None ,
                             err = e)

def get_a_req_and_render_by_requests_html(url , **kwargs) :
    fu = get_a_req_and_render_by_requests_html_async(url , **kwargs)
    return asyncio.run(fu)

async def get_a_rendered_html_and_save_async(url , fp , **kwargs) :
    fu = partial(get_a_req_and_render_by_requests_html_async , **kwargs)
    o = await fu(url)
    outer_html = o.html
    if outer_html :
        await write_txt_to_file_async(outer_html , fp)
    return o

async def get_rendered_htmls_and_save_async(urls , fps , **kwargs) :
    fu = partial(get_a_rendered_html_and_save_async , **kwargs)
    co_tasks = [fu(x , y) for x , y in zip(urls , fps)]
    return await asyncio.gather(*co_tasks)

def get_rendered_htmls_and_save_async_sync(urls , fps , **kwargs) :
    return asyncio.run(get_rendered_htmls_and_save_async(urls , fps , **kwargs))



"""

  """

import re
from datetime import datetime as dt
from datetime import time

from numpy import vectorize
from persiantools.jdatetime import JalaliDate

from .str import convert_digits_to_en


def persian_tools_jdate_from_int(jdate_int_fmt: {int , str}) :
    jd = jdate_int_fmt
    if not (isinstance(jd , str) or isinstance(jd , int)) :
        return jd

    jd = str(int(jd))
    int_fmt_jd = r'1[34]\d\d[0-2]\d[0-3]\d'
    cnd = re.fullmatch(int_fmt_jd , jd)

    if cnd is not None :
        return JalaliDate(int(jd[:4]) , int(jd[4 :6]) , int(jd[6 :8]))
    elif cnd is None :
        raise ValueError

def make_zero_padded_jdate_ie_iso_fmt(ist , sep = '/') :
    spl = ist.split(sep)
    for _i in range(1 , 3) :
        if int(spl[_i]) < 10 :
            spl[_i] = '0' + spl[_i]
    ou = '-'.join(spl)
    return ou

def make_datetime_from_iso_jdate_time(ist) :
    ptr = r'(\d{4})-(\d{2})-(\d{2})(\s*T?\s*)(\d{2}):(\d{2}):(\d{2})'
    cnd = re.fullmatch(ptr , ist)
    if cnd is None :
        raise ValueError

    jd = JalaliDate(int(cnd.group(1)) , int(cnd.group(2)) , int(cnd.group(3)))
    date = jd.to_gregorian()

    t = time(int(cnd.group(5)) , int(cnd.group(6)) , int(cnd.group(7)))

    date_time = dt.combine(date , t)
    return date_time

fu0 = make_datetime_from_iso_jdate_time
vect_make_datetime_from_iso_jdate_time = vectorize(fu0)

def find_jmonth_fr_df_col(df , targ_col , new_col , sep = '/') :
    s = df[targ_col].apply(convert_digits_to_en)

    pat = '(1[34]\d{2})' + sep + '(\d{2})' + sep + '\d{2}'
    _df = s.str.extract(pat)

    _df[new_col] = _df[0] + '-' + _df[1]

    df = df.join(_df[new_col])
    return df

def ex_all_jdate_fr_fa_str(s , sep = '/') :
    if not isinstance(s , str) :
        return s
    s = convert_digits_to_en(s)
    pat = '(1[34]\d{2})' + sep + '(\d{2})' + sep + '(\d{2})'
    return re.findall(pat , s)

def ex_1st_jdate_fr_fa_str(s , sep = '/') :
    ls = ex_all_jdate_fr_fa_str(s , sep)
    if isinstance(ls , list) and len(ls) > 0 :
        return ls[0]

def ex_1st_jmonth_fr_fa_str(s , sep = '/') :
    tu = ex_1st_jdate_fr_fa_str(s , sep)
    if isinstance(tu , tuple) :
        return tu[0] + '-' + tu[1]
